//Original by Timeslip //mods by harlanrm
Matrix TexProj;
//Matrix worldviewproj;
Matrix view;
Matrix proj;
Matrix world;
float4x4 MatrixPalette[4] : MATRIXPALETTE;
float3 SunVec;
float3 SunCol;
float3 EyePos;
float3 SunAmb;
float3 SkyCol;
float ticks;
float2 offset;
float TexBlend;
float2 rcpres;
int FogCol;
float3 FogCol2;
float FogStart;
float FogRange;
float BlendStart;
float BlendEnd;
float GrassDist;
float2 WindVec;
float PixelWidth;
float PixelHeight;

texture tex0;
texture tex1;
texture tex2;
texture tex3;
texture tex4;
sampler s0 = sampler_state { texture = <tex0>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=clamp; addressv = clamp; };
sampler s1 = sampler_state { texture = <tex1>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=wrap; addressv = wrap; };
sampler s2 = sampler_state { texture = <tex2>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=wrap; addressv = wrap; };
sampler s3 = sampler_state { texture = <tex3>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=wrap; addressv = wrap; };
sampler s4 = sampler_state { texture = <tex3>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=clamp; addressv = clamp; };

//For sampling the far water texture
sampler s5 = sampler_state { texture = <tex1>; minfilter = linear; magfilter = linear; mipfilter = none; addressu=wrap; addressv = wrap; };
sampler s6 = sampler_state { texture = <tex2>; minfilter = linear; magfilter = linear; mipfilter = none; addressu=wrap; addressv = wrap; };

sampler s7 = sampler_state { texture = <tex4>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=clamp; addressv = clamp; };

//static const float3 watercolour = { 0.9, 0.9, 1 };//original //wasn't used previously

static const float3 watercolour = { 0.4, 0.45, 0.5 };
//static const float FLT_MAX = 3.402823466e+38F;
static const float FLT_MAX = 340282346600000000000000000000000000000.0f;

/*
static const float4x4 TexProj = {
	0.5, 0.0, 0.0, 0.5,
	0.0, 0.5, 0.0, 0.5,
	0.0, 0.0, 0.5, 0.5,
	0.0, 0.0, 0.0, 1.0
};
*/

float4 RenderShader( in float2 TexCoords : TEXCOORD0, in float3 Light : TEXCOORD1, in float4 Height : TEXCOORD2) : COLOR0
{
	clip(Height);
	float3 Result = tex2D( s3, TexCoords) * Light;
	return float4(Result, 1);
}

struct StatVertIn {
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 texcoords : TEXCOORD0;
};

struct StatVertOut {
	float4 pos : POSITION;
	float2 texcoords : TEXCOORD0;
	float3 color : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float fog : TEXCOORD3;
	float4 screenpos: TEXCOORD4;
	float blend : TEXCOORD5;
};

StatVertOut StaticVS( StatVertIn IN ) {
	StatVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, world);
	float dist = length(worldpos.xyz - EyePos.xyz);
	OUT.fog = saturate( (FogRange - dist) / (FogRange-FogStart) );
	
	//Define % to Blend
	OUT.blend = saturate( (BlendEnd - dist) / (BlendEnd-BlendStart) );

	//Projection
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);
		
	//Texture Projection
	OUT.screenpos = mul( worldpos, TexProj );

	//Lighting
	OUT.normal = normalize( IN.normal * 2 - 1 ); //Decompress
	OUT.normal = normalize( mul(OUT.normal, (float3x3)world) ); //Rotate normals but don't translate them
	OUT.color = IN.color;
	
	return OUT;
}

struct StatInstVertIn {
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 texcoords : TEXCOORD0;
	float4 m1 : TEXCOORD1;
	float4 m2 : TEXCOORD2;
	float4 m3 : TEXCOORD3;
	float4 m4 : TEXCOORD4;
};

StatVertOut StaticInstVS( StatInstVertIn IN ) {
	StatVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Pack instance transform
    float4x4 worldmat = float4x4(IN.m1, IN.m2, IN.m3, IN.m4);

	//Define % to Fog
	float4 worldpos = mul(IN.pos, worldmat);
	OUT.fog = saturate( (FogRange - length(worldpos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Projection
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);
	
	//Lighting
	OUT.normal = normalize( mul(IN.normal, (float3x3)worldmat) );
	OUT.color = IN.color;
	
	OUT.screenpos = 0;
	OUT.blend = 0;
	
	return OUT;
}

float4 StaticPS( StatVertOut IN ) : COLOR0 {
	float4 Result = tex2D( s3, IN.texcoords);
	clip(Result.a - 0.5);
	
	//Lighting
	//float3 diffuse = saturate(dot(IN.normal,-SunVec)) * SunCol;
	//float3 ambient = SunAmb;
	
	//Result.rgb *= (diffuse + ambient) * IN.color;
	
	//Lighting
	float3 normal = normalize(IN.normal);
	Result.rgb *= saturate( (saturate(dot(-SunVec,normal)) * SunCol * IN.color) + (SunAmb * IN.color) );
	
	//Fogging
	Result.rgb = lerp(FogCol2, Result.rgb, IN.fog);
	
	//Blending
	float3 background = tex2Dproj(s0, IN.screenpos).rgb;
	Result.rgb = lerp( background, Result.rgb, IN.blend);

	return float4(Result.rgb, 1 - IN.fog);
}

struct StatRefVertOut {
	float4 pos : POSITION;
	float2 texcoords : TEXCOORD0;
	float3 color : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float fog : TEXCOORD3;
	float height : TEXCOORD6;
};

StatRefVertOut StaticReflectionVS( StatVertIn IN ) {
	StatRefVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, world);
	float dist = length(worldpos.xyz - EyePos.xyz);
	OUT.fog = saturate( (FogRange - dist) / (FogRange-FogStart) );

	//Projection
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);

	//Lighting
	OUT.normal = normalize( IN.normal * 2 - 1 ); //Decompress
	OUT.normal = normalize( mul(OUT.normal, (float3x3)world) ); //Rotate normals but don't translate them
	OUT.color = IN.color;
	
	//Figure hight of pixel in world coordinates
	OUT.height = worldpos.z;

	return OUT;
}

StatRefVertOut StaticReflectionInstVS( StatInstVertIn IN ) {
	StatRefVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Pack instance transform
    float4x4 worldmat = float4x4(IN.m1, IN.m2, IN.m3, IN.m4);

	//Define % to Fog
	float4 worldpos = mul(IN.pos, worldmat);
	OUT.fog = saturate( (FogRange - length(worldpos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Projection
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);
	
	//Lighting
	OUT.normal = normalize( mul(IN.normal, (float3x3)worldmat) );
	OUT.color = IN.color;
		
	//Figure height of pixel in world coordinates
	OUT.height = worldpos.z;
		
	return OUT;
}

float4 StaticReflectionPS( StatRefVertOut IN ) : COLOR0 {
	//Clip any pixels below a world height of zero
	clip(IN.height);
	
	float4 Result = tex2D( s3, IN.texcoords);
	clip(Result.a - 0.5);
	
	//Lighting
	//float3 diffuse = saturate(dot(IN.normal,-SunVec)) * SunCol;
	//float3 ambient = SunAmb;
	
	//Result.rgb *= (diffuse + ambient) * IN.color;
	
	//Lighting
	float3 normal = normalize(IN.normal);
	Result.rgb *= saturate( (saturate(dot(-SunVec,normal)) * SunCol * IN.color) + (SunAmb * IN.color) );
	
	//Fogging
	Result.rgb = lerp(FogCol2, Result.rgb, IN.fog);

	return float4(Result.rgb, 1);
}

struct LowestLandVertOut {
	float4 position : POSITION;
	float4 pos : TEXCOORD0;
	float2 texcoords : TEXCOORD1;
	float3 light : COLOR0;
};

LowestLandVertOut LowestLandVS( in float4 pos : POSITION ) {
	LowestLandVertOut OUT;
	
	float4 modpos = float4(pos.xy, -2080, pos.w);
	
	//OUT.position = mul( modpos, worldviewproj);
	OUT.position = mul(modpos, world);
	OUT.position = mul(OUT.position, view);
	OUT.position = mul(OUT.position, proj);
	
	OUT.pos = float4(modpos.xy - offset, modpos.zw);
	
	//Calculate texture coordinates
	OUT.texcoords = OUT.pos.xy/600;
	
	//Lighting
	float3 normal = float3(0,0,1);
	
	OUT.light = dot(normal,-SunVec) * SunCol;
	
	OUT.light = saturate(OUT.light) + saturate(SunAmb);
	
	return OUT;
}

float4 LowestLandPS( in LowestLandVertOut IN ) : COLOR0 {  
	//Define % to Fog
	float fog = saturate( (FogRange - length(IN.pos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	float3 Result = tex2D( s3, IN.texcoords).rgb;
	Result *= IN.light;
	
	//Fogging
	Result = lerp(FogCol2, Result, fog);
	
	return float4(Result, 1);
}

struct WaterVertOut {
	float4 position : POSITION;
	float4 pos : TEXCOORD0;
	float2 texcoord1 : TEXCOORD2;
	float2 texcoord2 : TEXCOORD3;
	float2 texcoord3 : TEXCOORD4;
	float4 screenpos : TEXCOORD5;
	float w : TEXCOORD6;
};

WaterVertOut WaterVS( in float4 pos : POSITION ) {
	WaterVertOut OUT;
	
	//OUT.position = mul( pos, worldviewproj);
	OUT.position = mul(pos, world);
	OUT.position = mul(OUT.position, view);
	OUT.position = mul(OUT.position, proj);
	OUT.w = OUT.position.w;

	OUT.screenpos = mul( pos, TexProj );
	
	OUT.pos = float4(pos.xy - offset, pos.zw);
	
	//Calculate various texture coordinates
	OUT.texcoord1 = OUT.pos.xy/6127;
	OUT.texcoord2 = OUT.pos.xy/1472;
	OUT.texcoord3 = OUT.pos.xy/892;
		
	return OUT;
}
float3 GetWaterNormal( in float2 texcoords, float blend ) : NORMAL {
	float3 normal = lerp( tex2D(s5,texcoords).rgb, tex2D(s6,texcoords).rgb, float3(blend,blend,blend) );
	return normalize( normal * 2 - 1 );
}

float3 GetFinalWaterNormal( float2 texcoord1, float2 texcoord2, float dist ) : NORMAL {
	//Blend together the normals from different sized areas of the same texture.
	float3 far_normal = GetWaterNormal( texcoord1, TexBlend );
	float3 close_normal = GetWaterNormal( texcoord2, TexBlend );
		                      
	return normalize( lerp( normalize(close_normal), normalize(far_normal), saturate(dist/8000) ) );
}

float4 ReflectionShader2( in WaterVertOut IN ) : COLOR0
{        
   //Define % to Fog
   float dist = length(IN.pos.xyz - EyePos.xyz);
   float fog = saturate( ( FogRange - dist ) / (FogRange-FogStart) );
      
   //Calculate eye vector
   float3 EyeVec = normalize(EyePos - IN.pos.xyz);
         
   //Sample depth texture
   float depth = tex2Dproj(s7, IN.screenpos).r - IN.w;
         
   float3 normal = GetFinalWaterNormal( IN.texcoord1, IN.texcoord2, dist );
   //float3 normal = float3(0, 0, 1 );
   
   //Reduce normal influence on perterbations near water's edge
   float3 normal2 = lerp( float3(0,0,1), normal, saturate(depth/50) );
   
   //Base the perturbation on the x and y direction of the normal
   float2 perturb = normal2.xy * 3;
   
   //fresnel factor
   float fresnel = dot(EyeVec,normal);

   //Sample the reflection texture   
   float3 Result = tex2Dproj( s0, IN.screenpos + float4(perturb*6,0,0) ) / 3;
   
   //Sample refraction texture
   depth = tex2Dproj(s7, IN.screenpos + float4(perturb.yx,0,0)).r - IN.w;;
   float3 Refract = tex2Dproj(s4, IN.screenpos + float4(perturb.yx,0,0));
    
   Refract = lerp( Refract, watercolour/5, saturate(depth/500) );
      
   Result = lerp( Result, Refract, saturate(fresnel*1.5) );
   
   //Ambient and Diffuse lighting
   Result = (saturate(dot(normal,-SunVec)) * Result * SunCol) + (Result * SunAmb);

   //Result *= watercolour;

   //Specular lighting
   float reflection = dot(EyeVec,reflect(SunVec,normal));
   float3 Specular = pow(reflection,40) * SunCol;

   //Additional Specular
   //Specular += pow(reflection,1000)*(1+4*fog);
   Specular *= pow((SunCol+SunCol.g-SunCol.b)+0.3,3);
   
   Result = Result + Specular / 2;

   //Fogging
   Result = lerp(FogCol2, Result, fog);
   
   //Distance
   //float alpha = saturate( (IN.w - 7168) / (FogRange - 7168) );

   return float4(Result, 1 - fog);
}

float4 NonReflectionShader( in WaterVertOut IN ) : COLOR0
{        
   //Define % to Fog
   float dist = length(IN.pos.xyz - EyePos.xyz);
   float fog = saturate( ( FogRange - dist ) / (FogRange-FogStart) );
   
   float3 normal = GetFinalWaterNormal( IN.texcoord1, IN.texcoord2, dist );
   
   //Base the perturbation on the x and y direction of the normal
   float2 perturb = normal.xy * 3;
   
   //Calculate eye vector
   float3 EyeVec = normalize(EyePos - IN.pos.xyz);
      
   //fresnel factor
   float fresnel = dot(EyeVec,normal);

   //Assume the sky color is the reflection color 
   float3 Result = SkyCol / 3;
   
   //Sample refraction texture
   float3 Refract = lerp( tex2Dproj(s4, IN.screenpos + float4(perturb,0,0)), watercolour, 0.3);
      
   Result = lerp( Result, Refract, fresnel );
   
   //Ambient and Diffuse lighting
   Result = (saturate(dot(normal,-SunVec)) * Result * SunCol) + (Result * SunAmb);

   //Result *= watercolour;

   //Specular lighting
   float reflection = dot(EyeVec,reflect(SunVec,normal));
   float3 Specular = pow(reflection,40) * SunCol;

   //Additional Specular
   //Specular += pow(reflection,1000)*(1+4*fog);
   Specular *= pow((SunCol+SunCol.g-SunCol.b)+0.3,3);
   
   Result = Result + Specular / 2;

   //Fogging
   Result = lerp(FogCol2, Result, fog);

   return float4(Result, 1);
}

float4 UnderWaterPS( in WaterVertOut IN ) : COLOR0
{
   //Define % to Fog
   float dist = length(IN.pos.xyz - EyePos.xyz);
   float fog = saturate( ( FogRange - dist ) / (FogRange-FogStart) );
   
   float3 normal = -GetFinalWaterNormal( IN.texcoord1, IN.texcoord2, dist );
   
   //Base the perturbation on the x and y direction of the normal
   float2 perturb = normal.xy * 5;
   
   //Calculate eye vector
   float3 EyeVec = normalize(EyePos - IN.pos.xyz);
      
   //Fresnel Factor
   //float fresnel = 1 - pow( 1 -dot(EyeVec,normal), 2);
   float fresnel = dot(EyeVec,normal);
         
   //Assume the sky color is the reflection color 
   float3 Result = SkyCol / 3;
   
   //Sample refraction texture
   float3 Refract = lerp( tex2Dproj(s4, IN.screenpos + float4(perturb,0,0)), watercolour, 0.2);
      
   Result = lerp( Result, Refract, fresnel );
   
   //Ambient and Diffuse lighting
   Result = (saturate(dot(-normal,-SunVec)) * SunCol * Result) + (Result * (SunAmb + watercolour));

   //Result *= watercolour;

   //Specular lighting
   float3 Specular = pow(dot(EyeVec,reflect(SunVec,normal)),40) * SunCol;

   //Additional Specular
   //Specular += pow(dot(EyeVec,reflect(SunVec,normal)),1000)*(1+4*fog);
   Specular *= pow((SunCol+SunCol.g-SunCol.b)+0.3,3);
   
   Result += Specular * 4;

   //Fogging
   Result = lerp(FogCol2, Result, fog);

   return float4(Result, 1);
   
   //return float4(tex2Dproj(s4, IN.screenpos + float4(perturb,0,0)).rgb, 1);
}

struct LandVertOut {
	float4 pos: POSITION;
	float2 texcoord: TEXCOORD1;
	float3 position: TEXCOORD2;
};

LandVertOut LandscapeVertexShader( float4 pos: POSITION, float2 texcoord: TEXCOORD0 ) {
	LandVertOut OUT;

	//Lower vertices that are close to the camera to ensure they are always below it
	float dist = length(pos.xyz - EyePos.xyz);
	float disp = lerp( 200, 0, saturate(dist/2000));
	pos.z -= disp;
	
	//OUT.pos = mul(pos, worldviewproj);
	OUT.pos = mul(pos, world);
	OUT.pos = mul(OUT.pos, view);
	OUT.pos = mul(OUT.pos, proj);
		
	OUT.texcoord = texcoord;
	OUT.position = pos;
	
	//Define % to Fog
	//OUT.fog = saturate( (FogRange - length(pos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );

	return OUT;
}
float4 LandscapePixelShader( in LandVertOut IN ) : COLOR0 {
	//Normalize incoming normal
	float3 normal = normalize(tex2D(s1,IN.texcoord).rgb * 2 - 1);
	
	//World Texture
	float3 color = tex2D(s3,IN.texcoord).rgb;
	
	//Detail Texture
	float detail = tex2D(s2,IN.texcoord * 1064).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 333).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 90).g + 0.5;
		
	//Lighting
	color *= detail * saturate( (saturate(dot(-SunVec,normal)) * SunCol) + SunAmb );
	
	//Define % to Fog
	float fog = saturate( (FogRange - length(IN.position.xyz - EyePos.xyz) ) / (FogRange-FogStart) );

	color = lerp(FogCol2, color, fog);
	
	return float4( color, 1 - fog );
}

struct LandRefVertOut {
	float4 pos: POSITION;
	float2 texcoord: TEXCOORD1;
	float fog: TEXCOORD2;
	float height: TEXCOORD3;
};
LandRefVertOut LandscapeRefVS( float4 pos: POSITION, float2 texcoord: TEXCOORD0 ) {
	LandRefVertOut OUT;

	//Lower vertices so there are fewer seams visible in the reflection
	pos.z -= 100;
	
	//OUT.pos = mul(pos, worldviewproj);
	OUT.pos = mul(pos, world);
	OUT.pos = mul(OUT.pos, view);
	OUT.pos = mul(OUT.pos, proj);	
	OUT.texcoord = texcoord;
	
	//Define % to Fog
	OUT.fog = saturate( (FogRange - length(pos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Define height of pixel for clipping to water plane
	OUT.height = pos.z;

	return OUT;
}
float4 LandscapeRefPS( in LandRefVertOut IN ) : COLOR0 {
	//Clip to water plane
	clip(IN.height);

	//Normalize incoming normal
	float3 normal = normalize(tex2D(s1,IN.texcoord).rgb * 2 - 1);
	
	//World Texture
	float3 color = tex2D(s3,IN.texcoord).rgb;
	
	//Detail Texture
	float detail = tex2D(s2,IN.texcoord * 1064).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 333).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 90).g + 0.5;
	
	//Lighting
	color *= detail * saturate( (saturate(dot(-SunVec,normal)) * SunCol) + SunAmb );
	
	//Fogging
	color = lerp(SkyCol, color, IN.fog);
	
	return float4(color,1);
}

struct GrassVertIn {
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 texcoords : TEXCOORD0;
};

struct GrassVertOut {
	float4 pos : POSITION;
	float2 texcoords : TEXCOORD0;
	float3 color : COLOR0;
	float fog : TEXCOORD1;
	float4 screenpos: TEXCOORD4;
	float blend : TEXCOORD5;
};

GrassVertOut GrassVS( GrassVertIn IN ) {
	GrassVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
		
	//Animate grass
	float4 worldpos = mul(IN.pos, world);

	float height = clamp(IN.pos.z, 0, 100) / 100;
	
	float2 wind = WindVec * 2.5 + 0.25;
		
	float2 pos_ticks1 = worldpos.xy / 1000 + ticks;
	float2 pos_ticks2 = worldpos.xy / 750 + ticks * 2;
	float2 pos_ticks3 = worldpos.xy / 500 + ticks * 3;
	float2 pos_ticks4 = worldpos.xy / 200 + ticks * 4;
	
	worldpos.xy += ( sin(pos_ticks1) + cos(pos_ticks2) + sin(pos_ticks3) + cos(pos_ticks4) ) * height * wind * 10;
	
	//Define % to Fog
	float dist = length(worldpos.xyz - EyePos.xyz);
	OUT.fog = saturate( (FogRange - dist) / (FogRange-FogStart) );
	
	//Define % to Blend
	OUT.blend = saturate( (BlendEnd - dist) / (BlendEnd-BlendStart) );

	//Projection
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);
	
	//Texture Projection
	OUT.screenpos = mul( worldpos, TexProj );
	
	//Lighting
	OUT.color = (SunCol * 0.25) + SunAmb;

	return OUT;
}

GrassVertOut GrassInstVS( StatInstVertIn IN ) {
	GrassVertOut OUT;
	
	//Pack instance transform
    float4x4 worldmat = float4x4(IN.m1, IN.m2, IN.m3, IN.m4);

	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, worldmat);
	float dist = length(worldpos.xyz - EyePos.xyz);
	OUT.fog = saturate( (FogRange - dist ) / (FogRange-FogStart) );
	OUT.blend = 1 - (saturate( (GrassDist-dist) / (GrassDist/5) ));
	OUT.blend += 0.4;
	OUT.screenpos = 0;
	
	//Animate grass
	float height = clamp(IN.pos.z, 0, 100) / 100;
	
	float2 wind = WindVec / 6;
	
	float2 pos_ticks1 = worldpos.xy / 1000 + ticks;
	float2 pos_ticks2 = worldpos.xy / 750 + ticks * 2;
	float2 pos_ticks3 = worldpos.xy / 500 + ticks * 3;
	float2 pos_ticks4 = worldpos.xy / 250 + ticks * 4;
	
	worldpos.xy += ( sin(pos_ticks1) + cos(pos_ticks2) + sin(pos_ticks3) + cos(pos_ticks4) ) * height * wind * 10;
	
	//Projection
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);
	
	//Lighting
	OUT.color = (SunCol * 0.25) + SunAmb;
	
	return OUT;
}

float4 GrassPS( GrassVertOut IN ) : COLOR0 {
	float4 Result = tex2D( s3, IN.texcoords);
	clip(Result.a - 0.4);
	
	//Lighting	
	Result.rgb *= IN.color;

	//Fogging
	Result.rgb = lerp(FogCol2, Result.rgb, IN.fog);
	
	//Blending
	float3 background = tex2Dproj(s0, IN.screenpos).rgb;
	Result.rgb = lerp( background, Result.rgb, IN.blend);

	return float4(Result.rgb, 1 - IN.fog);
}

struct RenderDepthVertIn {
	float4 pos: POSITION;
	float4 blendWeights: BLENDWEIGHT;
	float2 texcoords: TEXCOORD0;
};

struct RenderDepthVertOut {
	float4 pos: POSITION;
	float4 position: TEXCOORD0;
	//float4 col:  COLOR0;
	float2 texcoords: TEXCOORD1;
};

RenderDepthVertOut RenderDepthVS( in RenderDepthVertIn IN ) {
	RenderDepthVertOut OUT;
	
	//float4 worldpos = mul(IN.pos, MatrixPalette[0]);
		
	//float blendWeightsArray[4] = (float[4])IN.blendWeights;
	
	float4 worldpos = 0.0f;
	
	float4 blend = IN.blendWeights;
	
	if ( FogCol == 3 ) {
		blend[3] = 1 - blend[0] - blend[1] - blend[2];
	} else if ( FogCol == 2 ) {
		blend[2] = 1 - blend[0] - blend[1];
	} else if ( FogCol == 1 ) {
		blend[1] = 1 - blend[0];
	}

	//calculate pos
	if ( FogCol == 0 ) {
		//OUT.col = 0.5;
		worldpos = mul(IN.pos, MatrixPalette[0]);
	} else {
		//OUT.col = blend;
		if ( FogCol >= 1 ) {
			worldpos += mul(IN.pos, MatrixPalette[0]) * blend[0];
			worldpos += mul(IN.pos, MatrixPalette[1]) * blend[1];
		}
		if ( FogCol >= 2 ) {
			worldpos += mul(IN.pos, MatrixPalette[2]) * blend[2];
		}
		if ( FogCol >= 3 ) {
			worldpos += mul(IN.pos, MatrixPalette[3]) * blend[3];
		}
	}
	
	OUT.pos = mul(worldpos, view);
	OUT.pos = mul(OUT.pos, proj);
	
	OUT.position = OUT.pos;
	OUT.texcoords = IN.texcoords;
	
	return OUT;
}

float4 RenderDepthPS( RenderDepthVertOut IN ) : COLOR0 {
	//return IN.col;
	
	if ( TexBlend > 0.5 ) {
		//Apply alpha testing
		float alpha = tex2D( s1, IN.texcoords).a;
		clip(alpha - 0.5);
	}

	////Define % to Blend	
	//float blend = saturate( (BlendEnd - IN.position.w ) / (BlendEnd-BlendStart) );

	//float3 color = lerp( float3(1,1,1), float3(0,0,0), blend );
	
	return float4(IN.position.w,1,1,1);
}

struct BlendDepthVertIn {
	float4 pos : POSITION;
	float2 texcoord : TEXCOORD0;
};

struct ScreenVertOut {
	float4 pos: POSITION;
	float2 texcoord : TEXCOORD0;
};

ScreenVertOut ScreenQuadVS( in BlendDepthVertIn IN ) {
	ScreenVertOut OUT;
	
	OUT.pos = mul(IN.pos, world);
	OUT.pos = mul(OUT.pos, view);
	OUT.pos = mul(OUT.pos, proj);
	
	OUT.texcoord = IN.texcoord;
	
	return OUT;
}

float4 BlendDepthPS( ScreenVertOut IN ) : COLOR0 {
	//return float4( tex2D( s0, IN.texcoord ).rgb, 1);

	float3 color = tex2D( s4, IN.texcoord ).rgb;
	float depth = tex2D( s0, IN.texcoord ).r;
	
	//Display depth buffer for testing
	//depth = saturate(depth / FogRange);
	//return float4(depth,depth,depth, 1);
					
	//Define % to Blend	
	float blend = saturate( (BlendEnd - depth ) / (BlendEnd-BlendStart) );
	
	return float4(color, 1 - blend);
}

float4 ClearDepthPS( ScreenVertOut IN ) : COLOR0 {
	return float4(FLT_MAX/2,1,1,1);
}

//float blur_weights[4] = { 0.47442968, 0.23392642, 0.02804153, 0.00081723 };

float blur_weights[7] = { 0.199471, 0.176033, 0.120985, 0.064759, 0.026995, 0.008764, 0.002216 };

float2 TwelveKernel[12];

float4 HorizontalBlurPS( ScreenVertOut IN ) : COLOR0 {
	float4 Original = tex2D(s4, IN.texcoord);
    float3 Blurred = 0;
    float total_weight = 0;
    
    for(int i = 0; i < 12; i++)
    {
        float4 Current = tex2D(s4, IN.texcoord + TwelveKernel[i] );
        float weight = saturate(Original.a*Current.a);
        Blurred += Current.rgb * weight;
        total_weight += weight;
    }
    Blurred = (Blurred + Original.rgb) / (total_weight + 1.0);
            
    return float4(Blurred, Original.a);
}

float4 VerticalBlurPS( ScreenVertOut IN ) : COLOR0 {
	float4 orig_color = tex2D( s4, IN.texcoord );
			
	//return float4( orig_color.a, orig_color.a, orig_color.a, orig_color.a );
	
	float2 offset = float2( 0, PixelHeight );

	float4 color = 0;
	
	color += tex2D( s4, IN.texcoord - offset * 6 ) * blur_weights[6];
	color += tex2D( s4, IN.texcoord - offset * 5 ) * blur_weights[5];
	color += tex2D( s4, IN.texcoord - offset * 4 ) * blur_weights[4];
	color += tex2D( s4, IN.texcoord - offset * 3 ) * blur_weights[3];
	color += tex2D( s4, IN.texcoord - offset * 2 ) * blur_weights[2];
	color += tex2D( s4, IN.texcoord - offset ) * blur_weights[1];
	color += orig_color * blur_weights[0];
	color += tex2D( s4, IN.texcoord + offset ) * blur_weights[1];
	color += tex2D( s4, IN.texcoord + offset * 2 ) * blur_weights[2];
	color += tex2D( s4, IN.texcoord + offset * 3 ) * blur_weights[3];
	color += tex2D( s4, IN.texcoord + offset * 4 ) * blur_weights[4];
	color += tex2D( s4, IN.texcoord + offset * 5 ) * blur_weights[5];
	color += tex2D( s4, IN.texcoord + offset * 6 ) * blur_weights[6];
	
	float blur = saturate( color.a * orig_color.a );
	
	//Don't let the sky get too blurry
	if (orig_color.a <= 0.0001 ) {
		blur = max( 0.75, blur );
	}
	
	color.rgb = lerp(color.rgb, orig_color.rgb, blur );
		
	return float4( color.rgb, orig_color.a);
}

float4 CopyAndSetAlphaPS( ScreenVertOut IN ) : COLOR0 {
	float3 color = tex2D( s4, IN.texcoord ).rgb;

	return float4(color,1);
}

Technique T0
{
	
	//Used to render the reflected landscape
	Pass A
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;
		
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 LandscapeRefVS();
		PixelShader = compile ps_2_0 LandscapeRefPS();
	}
	//Used to render the landscape
	Pass B
	{
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		CullMode=CW;
		FogEnable=false;

		VertexShader = compile vs_2_0 LandscapeVertexShader();
		PixelShader = compile ps_2_0 LandscapePixelShader();
	}
	//Used to render the water in SM 2.0 mode
	Pass C
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		CullMode=CW;
		FogEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 ReflectionShader2();
	}
	//Used to render the water in SM 3.0 mode
	Pass D
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		CullMode=CW;
		FogEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 ReflectionShader2();
	}
	//Used for rendering distant statics
	Pass E
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=CW;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;
		
		VertexShader = compile vs_2_0 StaticVS();
		PixelShader = compile ps_2_0 StaticPS();
	}
	//Used for rendering the water from underneath
	Pass F
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		CullMode=CCW;
		FogEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 UnderWaterPS();
	}
	//Used for rendering water in non-reflective mode
	Pass G
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		CullMode=CW;
		FogEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 NonReflectionShader();
	}
	//Used for rendering reflected statics
	Pass H
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=CCW;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 StaticReflectionVS();
		PixelShader = compile ps_2_0 StaticReflectionPS();
	}
	//Used for rendering the lowest possible land out into infinity
	Pass I
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		CullMode=CW;
		FogEnable=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 LowestLandVS();
		PixelShader = compile ps_2_0 LowestLandPS();
	}
	//Used for rendering distant statics with HW instancing
	Pass J
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=CW;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;
		
		VertexShader = compile vs_3_0 StaticInstVS();
		PixelShader = compile ps_3_0 StaticPS();
	}
	//Used for rendering reflected distant statics with HW instancing
	Pass K
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=CCW;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 StaticReflectionInstVS();
		PixelShader = compile ps_2_0 StaticReflectionPS();
	}
	//Used for rendering grass
	Pass L
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 GrassVS();
		PixelShader = compile ps_2_0 GrassPS();
	}
	//Used for rendering grass with instancing
	Pass M
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_3_0 GrassInstVS();
		PixelShader = compile ps_3_0 GrassPS();
	}
	//Used for rendering the depth texture
	Pass N
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 RenderDepthVS();
		PixelShader = compile ps_2_0 RenderDepthPS();
	}
	//Used for blending what Morrowind draws with what MGE draws
	Pass O
	{
		AlphaBlendEnable=true;
		SRCBLEND = SRCALPHA;
		DESTBLEND = INVSRCALPHA;
		FogEnable=false;
		CullMode=None;
		ZEnable=false;
		ZWriteEnable=false;

		VertexShader = compile vs_2_0 ScreenQuadVS();
		PixelShader = compile ps_2_0 BlendDepthPS();
	}
	//Used for clearing the depth buffer to FLT_MAX
	Pass P
	{
		FogEnable=false;
		CullMode=None;
		ZEnable=false;
		ZWriteEnable=false;

		VertexShader = compile vs_2_0 ScreenQuadVS();
		PixelShader = compile ps_2_0 ClearDepthPS();
	}
	//Used for horizontal blurring
	Pass Q
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=false;
		ZWriteEnable=false;

		VertexShader = compile vs_2_0 ScreenQuadVS();
		PixelShader = compile ps_2_0 HorizontalBlurPS();
	}
	//Used for vertical blurring
	Pass R
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=false;
		ZWriteEnable=false;

		VertexShader = compile vs_2_0 ScreenQuadVS();
		PixelShader = compile ps_2_0 VerticalBlurPS();
	}
	//Used to copy a texture to the screen while also setting alpha to 0
	Pass S
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=false;
		ZWriteEnable=false;

		VertexShader = compile vs_2_0 ScreenQuadVS();
		PixelShader = compile ps_2_0 CopyAndSetAlphaPS();
	}
	//Used to set up the render state
	Pass Zero
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		FogColor=<FogCol>;
		ColorVertex=false;
		Lighting=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;
		StencilEnable=false;
		ColorWriteEnable=red|green|blue;
		CullMode=CCW;
		ShadeMode=gouraud;
		ColorOp[0]=selectarg1;
		ColorOp[1]=disable;
		AlphaOp[0]=disable;
		AlphaOp[1]=disable;
		ResultArg[0]=current;
		Texture[0]=null;
		Texture[1]=null;
		Texture[2]=null;
		Texture[3]=null;
		Texture[4]=null;
		Texture[5]=null;
		Texture[6]=null;
		Texture[7]=null;
		TexCoordIndex[0]=0;
		TexCoordIndex[1]=1;
		TexCoordIndex[2]=2;
		TexCoordIndex[3]=3;
		TexCoordIndex[4]=4;
		TexCoordIndex[5]=5;
		TexCoordIndex[6]=6;
		TexCoordIndex[7]=7;
		TextureTransformFlags[0]=0;
		TextureTransformFlags[1]=0;
		TextureTransformFlags[2]=0;
		TextureTransformFlags[3]=0;
		TextureTransformFlags[4]=0;
		TextureTransformFlags[5]=0;
		TextureTransformFlags[6]=0;
		TextureTransformFlags[7]=0;
		MagFilter[0]=Linear;
		MinFilter[0]=Linear;
		MipFilter[0]=Linear;
		Clipping=true;
	}
}